
library(exactci)
library(tools)
library(gnlm)
library(MASS)
PET=function(a1,a2,d1,d2){
  return(poisson.exact(c(a1,a2),c(d1,d2),alternative="two.sided")[[3]])
}

parOrig <- par()
par(bg=NA)

#basechange Positions
baseChange=read.table("MIX4_hg19_baseChange.txt",header=F,sep="\t",stringsAsFactors = F);colnames(baseChange)=c("CHROM","POS","REF","ALT")
bedFilter=read.table("181214_HG19_PCH_EZ_primary_target.bed",header=F,sep="\t",stringsAsFactors = F);colnames(bedFilter)=c("CHROM","START","END","GENE")
first=T
for (i in 1:nrow(bedFilter)){
  temp=data.frame(POS=as.integer(seq(bedFilter$START[i],bedFilter$END[i],1)))
  temp$CHROM=bedFilter$CHROM[i];temp$GENE=bedFilter$GENE[i]
  if (first){first=F;bf=temp}else{bf=rbind(bf,temp)}
}
#bf1=bf[! bf$POS %in% baseChange$V2,]

##For speed
files<-list.files(pattern = glob2rx("*_MD.vcf$"))
sl=unique(sapply(strsplit(files,"_"),"[[",1))

first=T
for (i in sl){
inputFile=list.files(pattern=glob2rx(paste0(i,"*IS*_MD.vcf")))
fileStuff=read.delim(inputFile,comment.char="#")
colnames(fileStuff)=c("CHROM","POS","id","REF","ALT","QUAL","FILTER","INFO","FORMAT","GT")
#fileStuff=merge(fileStuff,bf1,by=c("CHROM","POS"))
fileStuff$DP=as.integer(sapply(strsplit(fileStuff$GT,":"), "[[",2))
fileStuff$VD=as.integer(sapply(strsplit(fileStuff$GT,":"), "[[",3))
fileStuff$pmean=sapply(strsplit(fileStuff$INFO,";"), "[[",9)
fileStuff$pmean=as.numeric(sapply(strsplit(fileStuff$pmean,"="), "[[",2))
fileStuff$FILTER1="PASS"
fileStuff$FILTER1[fileStuff$QUAL<20]="QS"
fileStuff$FILTER1[! fileStuff$POS %in% bf$POS]="BED"
fileStuff$FILTER1[fileStuff$POS %in% baseChange$POS]="DN"
fileStuff$sample=i
if(first) {IS=fileStuff}else{IS=rbind(IS,fileStuff)}

inputFile=list.files(pattern=glob2rx(paste0(i,"*NT*_MD.vcf")))
fileStuff=read.delim(inputFile,comment.char="#")
colnames(fileStuff)=c("CHROM","POS","id","REF","ALT","QUAL","FILTER","INFO","FORMAT","GT")
#fileStuff=merge(fileStuff,bf1,by=c("CHROM","POS"))
fileStuff$DP=as.integer(sapply(strsplit(fileStuff$GT,":"), "[[",2))
fileStuff$VD=as.integer(sapply(strsplit(fileStuff$GT,":"), "[[",3))
fileStuff$pmean=sapply(strsplit(fileStuff$INFO,";"), "[[",9)
fileStuff$pmean=as.numeric(sapply(strsplit(fileStuff$pmean,"="), "[[",2))
fileStuff$FILTER1="PASS"
fileStuff$FILTER1[fileStuff$QUAL<20]="QS"
fileStuff$FILTER1[! fileStuff$POS %in% bf$POS]="BED"
fileStuff$FILTER1[fileStuff$POS %in% baseChange$POS]="DN"
fileStuff$sample=i
if(first) {NT=fileStuff;first=F}else{NT=rbind(NT,fileStuff)}

print (paste0(i," ",nrow(fileStuff[fileStuff$FILTER1 != "DN" & fileStuff$FILTER1 != "BED",]), " alts in exon non DS positions"))
print (paste0(i," ",length(unique(fileStuff$POS[fileStuff$FILTER1 != "DN" & fileStuff$FILTER1 != "BED"])), " exon non DS positions"))
}

#POOL IS and NT alts for bkgnd estimates
#temp=aggregate(DP~CHROM+POS+REF,data=IS[IS$FILTER1=="PASS" | IS$FILTER1=="QS",c("CHROM","POS","REF","DP")],sum) #will have multiple positions due to REFs
temp=aggregate(DP~CHROM+POS+REF,data=IS[IS$FILTER1=="PASS",c("CHROM","POS","REF","DP")],sum) #will have multiple positions due to REFs
ISPD=aggregate(DP~CHROM+POS,data=temp,max) #clean up extra rows.
ISPA=aggregate(cbind(DP,VD)~CHROM+POS+REF+ALT,data=IS[IS$FILTER1=="PASS",c("CHROM","POS","REF","ALT","DP","VD")],sum) #sum the alt depths at each position

#temp=aggregate(DP~CHROM+POS+REF,data=NT[NT$FILTER1=="PASS" | NT$FILTER1=="QS",c("CHROM","POS","REF","DP")],sum) #will have multiple positions due to REFs
temp=aggregate(DP~CHROM+POS+REF,data=NT[NT$FILTER1=="PASS",c("CHROM","POS","REF","DP")],sum) #will have multiple positions due to REFs
NTPD=aggregate(DP~CHROM+POS,data=temp,max) #clean up extra rows.
NTPA=aggregate(cbind(DP,VD)~CHROM+POS+REF+ALT,data=NT[NT$FILTER1=="PASS",c("CHROM","POS","REF","ALT","DP","VD")],sum) #sum the alt depths at each position

for (cs in sl){
  ##flipped
  IS1=IS[which( nchar(IS$REF)==1 & nchar(IS$ALT)==1 & IS$ALT!="." & IS$pmean>15 & IS$FILTER1=="PASS" & IS$FILTER=="PASS" & IS$sample==cs ),]
  if(nrow(IS1)==0){pet2=0;petco2=0}else{
    isnt=merge(IS1[,c("CHROM","POS","REF","ALT","DP","VD")],NTPA,by=c("CHROM","POS","REF","ALT"),all.x=T)
    temp=isnt[is.na(isnt$DP.y),c("CHROM","POS","REF","ALT","DP.x","VD.x")];colnames(temp)=c("CHROM","POS","REF","ALT","DP","VD.x")
    if(nrow(temp>0)){
      temp1=merge(temp,NTPD,c("CHROM","POS"),all.x=T);temp1$VD.y=0
      isnt=rbind(isnt[!is.na(isnt$DP.y),],temp1)
    }
    
    isnt$FILTER="."
    isnt$FILTER[is.na(isnt$DP.y)]="NoIS"; isnt$DP.y[is.na(isnt$DP.y)]=1
    isnt$VD.y[which(is.na(isnt$VD.y) )]=0
    
    # #NORMALISATION
    isnt$VD.n = as.integer(round(isnt$VD.y*6*(isnt$DP.x / isnt$DP.y),digits=0))
    isnt$DP.n = as.integer(round(isnt$DP.y*6*(isnt$DP.x / isnt$DP.y),digits=0))
    isnt$VD.n[isnt$DP.n > isnt$DP.y] = isnt$VD.y[isnt$DP.n > isnt$DP.y] #ensure dp.x*6 is always down sampling dp.y
    isnt$DP.n[isnt$DP.n > isnt$DP.y] = isnt$DP.y[isnt$DP.n > isnt$DP.y]
    
    isnt$VAF.x=isnt$VD.x/isnt$DP.x
    isnt$VAF.y=isnt$VD.n/isnt$DP.n
    #isnt$ALT.y[isnt$VD.y==0 ]="."
    #isnt=unique(isnt)
    
    isnt$PET=mapply(PET,isnt$VD.x,isnt$VD.n,isnt$DP.x,isnt$DP.n)
    isnt$PET[isnt$PET==0]=1e-100
    isnt$FILTER[isnt$POS %in% baseChange$V2 & isnt$FILTER =="."]="DN"
    isnt$FILTER[isnt$VAF.x<isnt$VAF.y/4 & isnt$FILTER =="."]="ISvaf" #PRETTY PLOTTING, AVOID REAL NT VARIANTS THAT OVERLAP IS VARIATNS
    petco2=0.05/nrow(IS1)
    isnt$PC=petco2
    isnt$FILTER[isnt$PET < petco2 & isnt$FILTER == "."]="PASS"
    isnt$FILTER[isnt$PET >= petco2 & isnt$FILTER == "."]="LOWPET"
    
    ISalts=paste0(nrow(isnt[isnt$FILTER=="PASS",]),"/",nrow(isnt))
    print(paste0(nrow(isnt)," VarDict PASSing alts"))
    print(paste0(nrow(isnt[isnt$FILTER=="PASS",]), " SNAQ-PASSing alts"))
    print(paste0(nrow(isnt)-nrow(isnt[isnt$FILTER=="PASS",]), " non-passing alts"))
    
    
    pet2=log10(isnt$PET[isnt$FILTER=="PASS" | isnt$FILTER=="LOWPET"]) #for plotting purposes need to remove odd scores
    
    temp=isnt[isnt$FILTER=="PASS" | isnt$FILTER=="LOWPET",]
    petco3=min(temp$PET)
    plot(log10(temp$VAF.y),log10(temp$VAF.x),xlim=c(-4,0),ylim=c(-4,0), main=paste0("IS for ",cs),
         pch=19,ylab="IS VAF (LOG)",xlab="NT VAF (LOG)",bg='transparent',
         cex=.7,col=colp[3+as.numeric(temp$FILTER =="PASS")])
    lines(c(-4,0),c(-4,0),type="l",lty=2,col=2)
    dev.copy(svg,'ISvsNTCorelation.svg',bg='transparent')
    dev.off()
    
    boxplot((isnt$DP.n/isnt$DP.x),ylim=c(0,10))
    write.table(isnt,file=paste0(cs,"-IS_Calls.txt"),quote = F, row.names = F,sep="\t")
  } #end of IS analysis

  #generate correlation plot
  #NT analysis all alts (except pmean, DN positions, QScore < 20, ALT.X = ALT.Y,SNV)
  NT1=NT[which( nchar(NT$REF)==1 & nchar(NT$ALT)==1 & NT$ALT!="." & NT$pmean>15 & NT$FILTER1=="PASS" & NT$sample == cs ),]
  ntis=merge(NT1[,c("CHROM","POS","REF","ALT","DP","VD")],ISPA,by=c("CHROM","POS","REF","ALT"))
  # #NORMALISATION
  ntis$VD.n = as.integer(round(ntis$VD.y*6*(ntis$DP.x / ntis$DP.y),digits=0))
  ntis$DP.n = as.integer(round(ntis$DP.y*6*(ntis$DP.x / ntis$DP.y),digits=0))
  ntis$VD.n[ntis$DP.n > ntis$DP.y] = ntis$VD.y[ntis$DP.n > ntis$DP.y] #ensure dp.x*6 is always down sampling dp.y
  ntis$DP.n[ntis$DP.n > ntis$DP.y] = ntis$DP.y[ntis$DP.n > ntis$DP.y]
  
  
  par(mgp=c(2.6,1,0))
  plot(log10(ntis$VD.n/ntis$DP.n),log10(ntis$VD.x/ntis$DP.x),xlim=c(-4,0),ylim=c(-4,0),
       ylab="NT VAF (Log)",xlab="IS VAF (Log)",main=cs,pch=19,cex=.7,col="#AFAFAF",bg='transparent',
       cex.lab=1.5,cex.axis=1.5)
  
  NT1=NT[which( nchar(NT$REF)==1 & nchar(NT$ALT)==1 & NT$ALT!="." & NT$pmean>15 & NT$FILTER1=="PASS" & NT$FILTER=="PASS" & NT$sample==cs),]
  #find REF-ALT match in ISPA, if no match grab DP from ISPD 
  isnt=merge(NT1[,c("CHROM","POS","REF","ALT","DP","VD")],ISPA,by=c("CHROM","POS","REF","ALT"),all.x=T)
  temp=isnt[is.na(isnt$DP.y),c("CHROM","POS","REF","ALT","DP.x","VD.x")];colnames(temp)=c("CHROM","POS","REF","ALT","DP","VD.x")
  temp1=merge(temp,ISPD,c("CHROM","POS"),all.x=T);temp1$VD.y=0
  isnt=rbind(isnt[!is.na(isnt$DP.y),],temp1)
  
   isnt$FILTER="."
   isnt$FILTER[is.na(isnt$DP.y)]="NoIS"; isnt$DP.y[is.na(isnt$DP.y)]=1
   isnt$VD.y[which(is.na(isnt$VD.y) )]=0
   
   # #NORMALISATION
   isnt$VD.n = as.integer(round(isnt$VD.y*6*(isnt$DP.x / isnt$DP.y),digits=0))
   isnt$DP.n = as.integer(round(isnt$DP.y*6*(isnt$DP.x / isnt$DP.y),digits=0))
   isnt$VD.n[isnt$DP.n > isnt$DP.y] = isnt$VD.y[isnt$DP.n > isnt$DP.y] #ensure dp.x*6 is always down sampling dp.y
   isnt$DP.n[isnt$DP.n > isnt$DP.y] = isnt$DP.y[isnt$DP.n > isnt$DP.y]
   
   isnt$VAF.x = isnt$VD.x / isnt$DP.x
   isnt$VAF.y = isnt$VD.n / isnt$DP.n
   isnt$PET=mapply(PET,isnt$VD.x,isnt$VD.n,isnt$DP.x,isnt$DP.n)
   isnt$PET[isnt$PET==0]=1e-100
   isnt$FILTER[isnt$POS %in% baseChange$V2 & isnt$FILTER =="."]="DN"
   isnt$FILTER[isnt$VAF.x<isnt$VAF.y/10 & isnt$FILTER =="."]="ISvaf" #PRETTY PLOTTING, AVOID REAL NT VARIANTS THAT OVERLAP IS VARIATNS
   petco1=0.05/nrow(NT1) #### ignored
   isnt$PC=petco3 #####NOTE THE CUTOFF IS SET BY MIN IS PET : petco3
   isnt$FILTER[isnt$PET < petco3 & isnt$FILTER == "."]="PASS"  #####NOTE THE CUTOFF IS SET BY MIN IS PET : petco3
   isnt$FILTER[isnt$PET >= petco3 & isnt$FILTER == "."]="LOWPET" #####NOTE THE CUTOFF IS SET BY MIN IS PET : petco3
   
   NTalts=paste0(nrow(isnt[isnt$FILTER=="PASS",]),"/",nrow(isnt))
   print(paste0(cs," ",nrow(isnt)," VarDict PASSing alts"))
   print(paste0(cs," ",nrow(isnt[isnt$FILTER=="PASS",]), " SNAQ-PASSing alts"))
   print(paste0(cs," ",nrow(isnt)-nrow(isnt[isnt$FILTER=="PASS",]), " non-passing alts"))
   
   pet1=log10(isnt$PET[isnt$FILTER=="PASS" | isnt$FILTER=="LOWPET"]) #remove oddballs from plot
   
   temp=isnt[isnt$FILTER=="PASS" | isnt$FILTER=="LOWPET",]#for plotting purposes need to remove odd scores
   
   colp=c("#afafaf","#99cb99","#E14040","#61cb61")
   points(log10(temp$VAF.y),log10(temp$VAF.x),xlim=c(-4,0),ylim=c(-4,0),
       pch=19, cex=.7,col=colp[3+as.numeric(temp$FILTER =="PASS")],
       cex.lab=1.5)
   lines(c(-4,0),c(-4,0),type="l",lty=2,col=2)
   dev.copy(svg,paste0(cs,'-NTvISCorelation.svg'),bg='transparent',width=5,height=5.4,pointsize=12)
   dev.off()
  
  
  write.table(isnt,file=paste0(cs,"-NT_Calls.txt"),quote = F, row.names = F,sep="\t")

pet1[pet1==0]=-0.0001;pet2[pet2==0]=-0.0001#clean split between NT and IS PET scores
pet=append(pet1,-pet2)
pet[pet< -50]=-50;pet[pet>50]=50
nrange=append(seq(as.integer(min(pet))-1,0,1),seq(1,as.integer(max(pet))+1,1))
colr=as.numeric(nrange < (log10(petco3)-2) | nrange>-log10(petco2)+1)# -2 is a hack to make sure bars are correctly colored
par(mgp=c(2.6,1,0))
c1 <- rgb(0xE1,0x40,0x40,max = 255, alpha = 80, names = "lt.blue")
c2 <- rgb(0x61,0xcb,0x61, max = 255, alpha = 80, names = "lt.pink")
hist1=hist(pet[pet<log10(petco3) | pet>-log10(petco2)],breaks=nrange,plot=F)
hist2=hist(pet[pet >= log10(petco3) & pet< -log10(petco2)],breaks=nrange,plot=F)
plot(hist1,col=c2,xlim=c(-50,10),ylim=c(0,30),border=1,xlab="PET Score (LOG)",main=cs,cex.axis=1.5,cex.lab=1.5)
plot(hist2,col=c1,add=T)
text(0,20,labels=ISalts);text(min(pet)+10,25,NTalts)

abline(v=0,col="blue")
dev.copy(svg,paste0(cs,'-PET_histogram.svg'),bg='transparent',width=5,height=5.4,pointsize=12)
dev.off()

}
# 
# hist.data=hist(pet,breaks=50,plot=F)
# hist.data$counts[hist.data$counts==0]=.1
# hist.data$counts=log10(hist.data$counts)
# plot(hist.data,ylim=c(-1,3),col=colr,border = "white",xlab="PET Score (LOG)",main=NULL)
# 
# hist1=hist(pet,breaks=50,ylim=c(0,40),xlim=c(-75,10),col=colr,border = "white",xlab="PET Score (LOG)",main=NULL)
# hist1=hist(pet,breaks=nrange,col=colr,border = "white",xlab="PET Score (LOG)",main=NULL)
# 
# hist1$counts[hist1$counts==0]=.1
# barplot(hist1$counts, log="y",col=colr,xlab=hist1$breaks,ylim=c(.1,600))

# colr=2*as.numeric(seq(-299.5,14.5,1)<log10(petco))+1
# hist1=hist(pet,breaks=seq(-300,15,1),xlim=c(-50,2),col=colr)
